@@ -1,5 +1,5 @@
 package App::Oozie::XML;
-$App::Oozie::XML::VERSION = '0.004';
+$App::Oozie::XML::VERSION = '0.005';
 use 5.010;
 use strict;
 use warnings;
@@ -360,6 +360,10 @@ sub _build_schema {
 
         # get the namespace and version
         my %attr      = map +( $_->name, $_->value ), $XML_SCHEMA->attributes;
+        if ( ! exists $attr{targetNamespace} ) {
+            # seems to be a change in new version
+            next;
+        }
         my $namespace = delete $attr{targetNamespace};
         my $version   = ( split /:/, $namespace )[-1];                       # assuming uri:oozie:...:$version
         my $prefix = ( split /:/, ( grep $attr{$_} eq $namespace, keys %attr )[0] )[-1];
@@ -422,7 +426,7 @@ App::Oozie::XML
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
